# ALEXIS 120. MCMC analyses of training data
# Save output as *.RData files
#
# MN 2020-01-28


## Clear everything ------------------------------------------------------------
rm(list = ls()) # clear memory
graphics.off()  # clear all plots
cat("\014")     # clear console (same as Ctrl-L in console)
## ----------------------------------------------------------------------------


## Load stuff ------------------------------------------------------------------
library(rstan)
library(HDInterval)  # hdi()

# Load MCMC data main experimenet, one object per participant
load("analys/mcmc_main.RData")  # Used for initial values for MCMC

# Load functions
source("analys/1_a120_functions.R") 
## -----------------------------------------------------------------------------


## Read raw data ---------------------------------------------------------------
d <- read.table("./rawdata/rawdata_a120_train.txt", sep = ",", header = TRUE,
                stringsAsFactors = FALSE)
d$llr <- ifelse(d$llr > 6, 6, d$llr)  # Fix max value (Python script set LLR > 6 to LLR = 6)
head(d)
table(d$id, d$task)
table(d$id, d$day)
## -----------------------------------------------------------------------------


## Get and save indiviual estimates (mdn, HPDI of posterior) -------------------

# Compile model 
stanmodel <- stan_model(file = "./analys/train.stan")

# Function to get samples from the posterior
sample_posterior <- function(id, fromday = 1, today = 3, init) {
  # Note init is list of list of initial values from previous fit, to help sampling
  
  g <- d[d$id == id & d$day <= today & d$day >= fromday , ] # Get data
  dat <- list(
    N <- nrow(g),
    N_cond <- length(unique(g$ici)),
    x <- g$llr,
    cond <- as.integer(factor(g$ici)),
    y <- g$pc) 
  
  
  model <- sampling(stanmodel, data = dat, chains = 1, iter = 1e4, warmup = 5e3, init = init)
  print(model)
  
  draws <- extract(model)
  draws
} 


# Function for deriving individual moving estimates based on day 1-3, 2-4, ..., 28-30
id_moving <- function(id){
  # Define moving time periods to be analyzed
  dd <- max(d$day[d$id == id])
  fromdd <- 1:(dd-2)  # First day of analysis
  todd <- fromdd + 2  # last day of analysis
   
  # Empty output matrix
  out <- matrix(numeric(16 * max(fromdd)), nrow = max(fromdd))
  colnames(out) <- c('day', 'm6_mdn', 'm12_mdn', 'm24_mdn', 'm48_mdn', 'm200_mdn',
                            'm6_lo95', 'm12_lo95', 'm24_lo95', 'm48_lo95', 'm200_lo95',
                            'm6_hi95', 'm12_hi95', 'm24_hi95', 'm48_hi95', 'm200_hi95')
  out <- data.frame(out)  # Data frame with zeros
  
  # Inital values for first day (from on pre-test data)
  fname <- ls(envir=.GlobalEnv, pattern = paste('maindraws_id', id, sep = ''))
  print(fname)  # Just as a check
  draws <- get(fname)
  m <- apply(draws$m, 2, median)[c(9, 11, 13, 15, 16)]
  w <- apply(draws$w, 2, median)[c(9, 11, 13, 15, 16)]
  lambda <- apply(draws$lambda, 2, median)[2]
  init <- list(list(m = m, w = w, lambda = lambda)) # Initial inital values!
  
  
  # Loop over time-periods (fromday-today)
  for (j in 1:length(fromdd)){
    # Obtain samples from posterior
    draws <- sample_posterior(id = id, fromday = fromdd[j], today  = todd[j], init = init)
    
    # Calculate summaries of posterior, to be saved and/or be used as initial values for next mcmc
    m <- apply(draws$m, 2, median)                     # Median posterior
    m_lo <- apply(draws$m, 2, hdi, credMass = 0.95)[1, ]  # Lo 95 % credibility
    m_hi <- apply(draws$m, 2, hdi, credMass = 0.95)[2, ]  # Hi 95 % credibility
    w <- apply(draws$w, 2, median)                     # Spread, for initial values
    lambda <- median(draws$lambda)     # Lambda, for initial values(single value for all conditions)
    init <- list(list(m = m, w = w, lambda = lambda))  # New initial values for sampling
    
    out[j, ] <- c(j, m, m_lo, m_hi)  # Add data to row j of dataframe
  }
  
  out
}

# Get and save individual data
train03 <- id_moving(120003)
train09 <- id_moving(120009)
train11 <- id_moving(120011)
train12 <- id_moving(120012)

save(train03, train09, train11, train12, file = "analys/mcmc_train.RData")
## -----------------------------------------------------------------------------

